import { echo } from "@atums/echo";
import {
	createDefaultGuildData,
	errorMessages,
	httpStatus,
} from "#environment/constants";
import { cassandra } from "#lib/database";
import { pika } from "#lib/utils";
import { isValidGuildName } from "#lib/validation/guild/name";

import type { UserSession } from "#types/config";
import type {
	CreateGuildRequest,
	CreateGuildResponse,
	ExtendedRequest,
	GuildResponse,
	GuildRow,
	RouteDef,
} from "#types/server";

const routeDef: RouteDef = {
	method: "POST",
	accepts: "application/json",
	returns: "application/json",
	needsBody: "json",
	needsAuth: true,
};

async function handler(request: ExtendedRequest): Promise<Response> {
	try {
		const session = request.session as UserSession;
		const { name, description } = request.requestBody as CreateGuildRequest;

		if (!name) {
			const response: CreateGuildResponse = {
				code: httpStatus.BAD_REQUEST,
				success: false,
				error: "Guild name is required",
			};
			return Response.json(response, { status: httpStatus.BAD_REQUEST });
		}

		const nameValidation = isValidGuildName(name);
		if (!nameValidation.valid || !nameValidation.value) {
			const response: CreateGuildResponse = {
				code: httpStatus.BAD_REQUEST,
				success: false,
				error: nameValidation.error || "Invalid guild name",
			};
			return Response.json(response, { status: httpStatus.BAD_REQUEST });
		}

		const existingGuildQuery = `
			SELECT guild_id FROM guilds_by_owner
			WHERE owner_id = ? AND name = ?
			LIMIT 1
		`;
		const existingGuildResult = (await cassandra.execute(existingGuildQuery, [
			session.id,
			nameValidation.value,
		])) as { rows: Array<{ guild_id: string }> };

		if (existingGuildResult.rows.length > 0) {
			const response: CreateGuildResponse = {
				code: httpStatus.CONFLICT,
				success: false,
				error: "You already own a guild with this name",
			};
			return Response.json(response, { status: httpStatus.CONFLICT });
		}

		const guildId = pika.gen("guild");
		const defaultData = createDefaultGuildData();
		const now = new Date();

		const insertGuildQuery = `
			INSERT INTO guilds (
				id, name, description, icon_url, banner_url, splash_url,
				owner_id, verification_level, default_message_notifications,
				system_channel_id, rules_channel_id, public_updates_channel_id,
				features, preferred_locale, created_at, updated_at
			) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
		`;

		const insertParams = [
			guildId,
			nameValidation.value,
			description || null,
			null, // icon_url
			null, // banner_url
			null, // splash_url
			session.id,
			Number(defaultData.verification_level),
			Number(defaultData.default_message_notifications),
			null, // system_channel_id
			null, // rules_channel_id
			null, // public_updates_channel_id
			defaultData.features,
			defaultData.preferred_locale,
			now,
			now,
		];

		await cassandra.execute(insertGuildQuery, insertParams, { prepare: true });

		const insertOwnerQuery = `
			INSERT INTO guilds_by_owner (
				owner_id, guild_id, name, created_at
			) VALUES (?, ?, ?, ?)
		`;

		await cassandra.execute(
			insertOwnerQuery,
			[session.id, guildId, nameValidation.value, now],
			{ prepare: true },
		);

		const insertChronologicalQuery = `
			INSERT INTO guilds_by_owner_chronological (
				owner_id, created_at, guild_id, name
			) VALUES (?, ?, ?, ?)
		`;

		await cassandra.execute(
			insertChronologicalQuery,
			[session.id, now, guildId, nameValidation.value],
			{ prepare: true },
		);

		const guildQuery = `
			SELECT id, name, description, icon_url, banner_url, splash_url,
				   owner_id, verification_level, default_message_notifications,
				   system_channel_id, rules_channel_id, public_updates_channel_id,
				   features, preferred_locale, created_at, updated_at
			FROM guilds WHERE id = ? LIMIT 1
		`;

		const guildResult = (await cassandra.execute(guildQuery, [guildId], {
			prepare: true,
		})) as {
			rows: Array<GuildRow>;
		};

		const guild = guildResult.rows[0];
		if (!guild) {
			const response: CreateGuildResponse = {
				code: httpStatus.INTERNAL_SERVER_ERROR,
				success: false,
				error: "Failed to fetch created guild",
			};
			return Response.json(response, {
				status: httpStatus.INTERNAL_SERVER_ERROR,
			});
		}

		const responseGuild: GuildResponse = {
			id: guild.id,
			name: guild.name,
			description: guild.description,
			iconUrl: guild.icon_url,
			bannerUrl: guild.banner_url,
			splashUrl: guild.splash_url,
			ownerId: guild.owner_id,
			verificationLevel: guild.verification_level,
			defaultMessageNotifications: guild.default_message_notifications,
			systemChannelId: guild.system_channel_id,
			rulesChannelId: guild.rules_channel_id,
			publicUpdatesChannelId: guild.public_updates_channel_id,
			features: Array.from(guild.features || []),
			preferredLocale: guild.preferred_locale,
			createdAt: guild.created_at.toISOString(),
			updatedAt: guild.updated_at.toISOString(),
		};

		const response: CreateGuildResponse = {
			code: httpStatus.CREATED,
			success: true,
			message: "Guild created successfully",
			guild: responseGuild,
		};

		return Response.json(response, { status: httpStatus.CREATED });
	} catch (error) {
		echo.error({
			message: "Error creating guild",
			error,
			userId: request.session?.id,
		});

		const response: CreateGuildResponse = {
			code: httpStatus.INTERNAL_SERVER_ERROR,
			success: false,
			error: errorMessages.INTERNAL_SERVER_ERROR,
		};
		return Response.json(response, {
			status: httpStatus.INTERNAL_SERVER_ERROR,
		});
	}
}

export { handler, routeDef };
