import { echo } from "@atums/echo";
import { errorMessages, httpStatus } from "#environment/constants";
import { cassandra } from "#lib/database";

import type { UserSession } from "#types/config";
import type {
	BaseResponse,
	DeleteGuildRequest,
	ExtendedRequest,
	RouteDef,
} from "#types/server";

const routeDef: RouteDef = {
	method: "DELETE",
	accepts: "application/json",
	returns: "application/json",
	needsBody: "json",
	needsAuth: true,
};

async function handler(request: ExtendedRequest): Promise<Response> {
	try {
		const session = request.session as UserSession;
		const { id, name } = request.requestBody as DeleteGuildRequest;

		if (!id || !name) {
			const response: BaseResponse = {
				code: httpStatus.BAD_REQUEST,
				success: false,
				error: "Both guild ID and name are required for confirmation",
			};
			return Response.json(response, { status: httpStatus.BAD_REQUEST });
		}

		if (!id.startsWith("guild_")) {
			const response: BaseResponse = {
				code: httpStatus.BAD_REQUEST,
				success: false,
				error: "Invalid guild ID format",
			};
			return Response.json(response, { status: httpStatus.BAD_REQUEST });
		}

		const guildQuery = `
			SELECT id, name, owner_id, created_at
			FROM guilds WHERE id = ? LIMIT 1
		`;

		const guildResult = (await cassandra.execute(guildQuery, [id], {
			prepare: true,
		})) as {
			rows: Array<{
				id: string;
				name: string;
				owner_id: string;
				created_at: Date;
			}>;
		};

		if (!guildResult?.rows || guildResult.rows.length === 0) {
			const response: BaseResponse = {
				code: httpStatus.NOT_FOUND,
				success: false,
				error: "Guild not found",
			};
			return Response.json(response, { status: httpStatus.NOT_FOUND });
		}

		const guild = guildResult.rows[0];
		if (!guild) {
			const response: BaseResponse = {
				code: httpStatus.NOT_FOUND,
				success: false,
				error: "Guild not found",
			};
			return Response.json(response, { status: httpStatus.NOT_FOUND });
		}

		if (guild.owner_id !== session.id) {
			const response: BaseResponse = {
				code: httpStatus.FORBIDDEN,
				success: false,
				error: "You can only delete guilds that you own",
			};
			return Response.json(response, { status: httpStatus.FORBIDDEN });
		}

		if (guild.name !== name) {
			const response: BaseResponse = {
				code: httpStatus.BAD_REQUEST,
				success: false,
				error: `Guild name confirmation failed. Expected "${guild.name}" but got "${name}". Please ensure the name matches exactly.`,
			};
			return Response.json(response, { status: httpStatus.BAD_REQUEST });
		}

		const deleteGuildQuery = "DELETE FROM guilds WHERE id = ?";
		await cassandra.execute(deleteGuildQuery, [id], { prepare: true });

		const deleteOwnerQuery =
			"DELETE FROM guilds_by_owner WHERE owner_id = ? AND name = ?";
		await cassandra.execute(deleteOwnerQuery, [session.id, guild.name], {
			prepare: true,
		});

		const deleteChronologicalQuery = `
			DELETE FROM guilds_by_owner_chronological
			WHERE owner_id = ? AND created_at = ? AND guild_id = ?
		`;
		await cassandra.execute(
			deleteChronologicalQuery,
			[session.id, guild.created_at, id],
			{ prepare: true },
		);

		// TODO: delete related data like channels, members, messages when those features are implemented

		const response: BaseResponse = {
			code: httpStatus.OK,
			success: true,
			message: `Guild "${guild.name}" has been successfully deleted`,
		};

		return Response.json(response, { status: httpStatus.OK });
	} catch (error) {
		echo.error({
			message: "Error deleting guild",
			error,
			userId: request.session?.id,
			guildId: (request.requestBody as DeleteGuildRequest)?.id,
		});

		const response: BaseResponse = {
			code: httpStatus.INTERNAL_SERVER_ERROR,
			success: false,
			error: errorMessages.INTERNAL_SERVER_ERROR,
		};
		return Response.json(response, {
			status: httpStatus.INTERNAL_SERVER_ERROR,
		});
	}
}

export { handler, routeDef };
