import { echo } from "@atums/echo";
import { errorMessages, httpStatus } from "#environment/constants";
import { cassandra } from "#lib/database";

// import type { UserSession } from "#types/config";
import type {
	BaseResponse,
	ExtendedRequest,
	GuildResponse,
	GuildRow,
	RouteDef,
} from "#types/server";

interface GuildGetResponse extends BaseResponse {
	guild?: GuildResponse;
}

const routeDef: RouteDef = {
	method: "GET",
	accepts: "*/*",
	returns: "application/json",
};

async function handler(request: ExtendedRequest): Promise<Response> {
	try {
		// const session = request.session as UserSession;
		const { id: guildId } = request.params;

		if (!guildId) {
			const response: GuildGetResponse = {
				code: httpStatus.BAD_REQUEST,
				success: false,
				error: "Guild ID is required",
			};
			return Response.json(response, { status: httpStatus.BAD_REQUEST });
		}

		const guildQuery = `
			SELECT id, name, description, icon_url, banner_url, splash_url,
				   owner_id, verification_level, default_message_notifications,
				   system_channel_id, rules_channel_id, public_updates_channel_id,
				   features, preferred_locale, created_at, updated_at
			FROM guilds WHERE id = ? LIMIT 1
		`;

		const guildResult = (await cassandra.execute(guildQuery, [guildId], {
			prepare: true,
		})) as {
			rows: Array<GuildRow>;
		};

		if (!guildResult?.rows || guildResult.rows.length === 0) {
			const response: GuildGetResponse = {
				code: httpStatus.NOT_FOUND,
				success: false,
				error: "Guild not found",
			};
			return Response.json(response, { status: httpStatus.NOT_FOUND });
		}

		const guild = guildResult.rows[0];
		if (!guild) {
			const response: GuildGetResponse = {
				code: httpStatus.NOT_FOUND,
				success: false,
				error: "Guild not found",
			};
			return Response.json(response, { status: httpStatus.NOT_FOUND });
		}

		// TODO: In the future, check if user has permission to view this guild
		// const isOwner = session?.id === guild.owner_id;

		const responseGuild: GuildResponse = {
			id: guild.id,
			name: guild.name,
			description: guild.description,
			iconUrl: guild.icon_url,
			bannerUrl: guild.banner_url,
			splashUrl: guild.splash_url,
			ownerId: guild.owner_id,
			verificationLevel: guild.verification_level,
			defaultMessageNotifications: guild.default_message_notifications,
			systemChannelId: guild.system_channel_id,
			rulesChannelId: guild.rules_channel_id,
			publicUpdatesChannelId: guild.public_updates_channel_id,
			features: Array.from(guild.features || []),
			preferredLocale: guild.preferred_locale,
			createdAt: guild.created_at.toISOString(),
			updatedAt: guild.updated_at.toISOString(),
		};

		const response: GuildGetResponse = {
			code: httpStatus.OK,
			success: true,
			message: "Guild retrieved successfully",
			guild: responseGuild,
		};

		return Response.json(response, { status: httpStatus.OK });
	} catch (error) {
		echo.error({
			message: "Error fetching guild",
			error,
			guildId: request.params.id,
			userId: request.session?.id,
		});

		const response: GuildGetResponse = {
			code: httpStatus.INTERNAL_SERVER_ERROR,
			success: false,
			error: errorMessages.INTERNAL_SERVER_ERROR,
		};
		return Response.json(response, {
			status: httpStatus.INTERNAL_SERVER_ERROR,
		});
	}
}

export { handler, routeDef };
