import { echo } from "@atums/echo";
import { errorMessages, httpStatus } from "#environment/constants";
import { cassandra } from "#lib/database";

import type { UserSession } from "#types/config";
import type {
	ExtendedRequest,
	GuildListResponse,
	GuildResponse,
	GuildRow,
	RouteDef,
} from "#types/server";

const routeDef: RouteDef = {
	method: "GET",
	accepts: "*/*",
	returns: "application/json",
	needsAuth: true,
};

async function handler(request: ExtendedRequest): Promise<Response> {
	try {
		const session = request.session as UserSession;
		const { limit = "10", offset = "0" } = request.query;

		const limitNum = Math.min(
			Math.max(Number.parseInt(limit, 10) || 10, 1),
			50,
		);
		const offsetNum = Math.max(Number.parseInt(offset, 10) || 0, 0);

		const guildIdsQuery = `
			SELECT guild_id, name, created_at
			FROM guilds_by_owner_chronological
			WHERE owner_id = ?
			LIMIT ?
		`;

		const guildIdsResult = (await cassandra.execute(
			guildIdsQuery,
			[session.id, limitNum + offsetNum],
			{ prepare: true },
		)) as {
			rows: Array<{
				guild_id: string;
				name: string;
				created_at: Date;
			}>;
		};

		if (!guildIdsResult?.rows || guildIdsResult.rows.length === 0) {
			const response: GuildListResponse = {
				code: httpStatus.OK,
				success: true,
				message: "No guilds found",
				guilds: [],
				count: 0,
			};
			return Response.json(response, { status: httpStatus.OK });
		}

		const relevantGuildIds = guildIdsResult.rows
			.slice(offsetNum, offsetNum + limitNum)
			.map((row) => row.guild_id);

		if (relevantGuildIds.length === 0) {
			const response: GuildListResponse = {
				code: httpStatus.OK,
				success: true,
				message: "No more guilds found",
				guilds: [],
				count: 0,
			};
			return Response.json(response, { status: httpStatus.OK });
		}

		const guildDetailsQuery = `
			SELECT id, name, description, icon_url, banner_url, splash_url,
				   owner_id, verification_level, default_message_notifications,
				   system_channel_id, rules_channel_id, public_updates_channel_id,
				   features, preferred_locale, created_at, updated_at
			FROM guilds WHERE id IN (${relevantGuildIds.map(() => "?").join(", ")})
		`;

		const guildsResult = (await cassandra.execute(
			guildDetailsQuery,
			relevantGuildIds,
			{
				prepare: true,
			},
		)) as {
			rows: Array<GuildRow>;
		};

		const guildsMap = new Map<string, GuildResponse>();

		for (const guild of guildsResult.rows) {
			guildsMap.set(guild.id, {
				id: guild.id,
				name: guild.name,
				description: guild.description,
				iconUrl: guild.icon_url,
				bannerUrl: guild.banner_url,
				splashUrl: guild.splash_url,
				ownerId: guild.owner_id,
				verificationLevel: guild.verification_level,
				defaultMessageNotifications: guild.default_message_notifications,
				systemChannelId: guild.system_channel_id,
				rulesChannelId: guild.rules_channel_id,
				publicUpdatesChannelId: guild.public_updates_channel_id,
				features: Array.from(guild.features || []),
				preferredLocale: guild.preferred_locale,
				createdAt: guild.created_at.toISOString(),
				updatedAt: guild.updated_at.toISOString(),
			});
		}

		const orderedGuilds: GuildResponse[] = [];
		for (const guildId of relevantGuildIds) {
			const guild = guildsMap.get(guildId);
			if (guild) {
				orderedGuilds.push(guild);
			}
		}

		const response: GuildListResponse = {
			code: httpStatus.OK,
			success: true,
			message: `Found ${orderedGuilds.length} guild(s)`,
			guilds: orderedGuilds,
			count: orderedGuilds.length,
		};

		return Response.json(response, { status: httpStatus.OK });
	} catch (error) {
		echo.error({
			message: "Error fetching user guilds",
			error,
			userId: request.session?.id,
		});

		const response: GuildListResponse = {
			code: httpStatus.INTERNAL_SERVER_ERROR,
			success: false,
			error: errorMessages.INTERNAL_SERVER_ERROR,
		};
		return Response.json(response, {
			status: httpStatus.INTERNAL_SERVER_ERROR,
		});
	}
}

export { handler, routeDef };
